#!/usr/bin/perl

my $Version = '20090122';

if ($ARGV[0] eq '-V')
{
	die "$Version\n";
};

my $acl_type = $ARGV[0];

use SQLayer;
use strict;
use DaemonLib;

if (($> == 0) or ($< == 0))
{
	die "Do not run me as root! Its dangerous.\n";
};

# Disable output buffering
$|=1;           

$SIG{TERM} = 'self_destroy';
$SIG{KILL} = 'self_destroy';
$SIG{INT} = 'self_destroy';
$SIG{QUIT} = 'self_destroy';
$SIG{ABRT} = 'self_destroy';

my $DLib = DaemonLib->new
(
	logfacility 	=> 'EXT-ACL!'.uc($acl_type)
);

my %config; # = $DLib->get_config_from_file("/usr/local/etc/sparrow.conf"); 
dbmopen(%config, "/var/db/sparrow/config", 0660)
	or $DLib->dielog("Cant open config hash (/var/db/sparrow/config): $!");

# there is nothing harmful if no debug defined by default
$DLib->debug($config{'debug'});

$DLib->dielog("Logfile not defined!") unless $config{'logfile'};
$DLib->logfile($config{'logfile'});

my $D = SQLayer -> new
(
        database 	=> 'DBI:Pg:dbname='.$config{'dbname'}.';host='.$config{'dbhost'},
        user 		=> $config{'dbuser'},
        password 	=> $config{'dbpassword'}
);
$DLib->dielog("Database ".$config{'dbname'}." connection failed!".$D->errstr) 
	unless $D->{'dbh'}->ping;
$D->DEBUG(1) if $config{'debug'};

my $sql_query;
if ($acl_type eq 'user_access')
{
	$sql_query = "SELECT cache_access FROM clients WHERE is_workstation = false AND id = ";
	$config{'def_user_access'} = 1;
#	$config{'def_ip_access'} = 0;
}
elsif ($acl_type eq 'ip_access')
{
	$sql_query = "SELECT cache_access FROM clients WHERE is_workstation = true AND id = ";
	$config{'def_ip_access'} = 1;
#	$config{'def_user_access'} = 0;
}
elsif ($acl_type eq 'blacklist')
{
	$sql_query = "SELECT true FROM sitelist WHERE is_black = true AND site = ";
}
elsif ($acl_type eq 'whitelist')
{
	$sql_query = "SELECT true FROM sitelist WHERE is_black = false AND site = ";
}
else
{
	die "Unknown acl_type: $acl_type!\n".
		"Usage: $0 blacklist\n".
		"       $0 whitelist\n".
		"       $0 ip_access\n".
		"       $0 user_access\n";
};

dbmclose(%config);

# Usually squid starts multiple copies of external_acl helper, so 
# it would be nice to see PID in log.
$DLib->include_pid_in_log(1);

$DLib->writelog("NOTICE Ready to serve squid queries.");

# Read squid's access query 
while (<STDIN>) 
{
	chomp;

	# refuse all queries, if squidAgent doesnt run within 5 minutes (300 sec)
	dbmopen(%config, "/var/db/sparrow/config", 0660)
		or $DLib->dielog("Cant open config hash (/var/db/sparrow/config): $!");
	if (($config{'squid_log_agent_lastrun'} + 300) < time)
	{
		$DLib->debuglog("SquidAgent last run over 5 minutes ago (".
			scalar(localtime($config{'squid_log_agent_lastrun'})).")!");
		print "ERR\n";
		dbmclose(%config);
		next;
	};
	dbmclose(%config);

	my ($param) = $DLib->extract_shellwords($_);
	if ($D -> row($sql_query.$D->quote($param).";"))
	{
		$DLib->debuglog("Reply positive for $param.");
		print "OK\n";
	}
	else
	{
		$DLib->debuglog("Reply negative for $param.");
		print "ERR\n";
	}	
}

sub self_destroy
{
        $DLib->writelog("NOTICE Exiting via SIGnal.");
        exit;
}
