#!/usr/bin/perl

# Sparrow Periodic maintenance. Run from cron daily.

my $Version = '20090122';

if ($ARGV[0] eq '-V')
{
	die "$Version\n";
};

use SQLayer;
use strict;
use POSIX qw(strftime);
use DaemonLib;

# rotate_needed	(
#	cur_time => time, 
#	max_period_size => 604800, 
#	period_started_at => 1231325477
# );
sub rotate_needed
{
	my %HKeys = @_;
	my $cur_time 		= $HKeys{'cur_time'};
	my $period_started_at 	= $HKeys{'period_started_at'};
	my $max_period_size 	= $HKeys{'max_period_size'};

	# size is in seconds.
	my $cur_period_size = $cur_time - $period_started_at;
	unless ($max_period_size)
	{
		# size is not set.
		# this means, that we should rotate every month.
		my $period_start_month = (localtime($period_started_at))[4];
		my $cur_month = (localtime($cur_time))[4];
		# this should do the trick at most cases (rotate appears, when 
		# month changes - usually at 1 date).
	 	return 1 if $period_start_month != $cur_month;
		# this should work in all other (not common) cases.
		# for example: when this script was not ran for a year,
		# so month remains the same (but year has changed).
		return 1 if $cur_period_size > 2592000; # 30 days in seconds
	}
	else
	{
		return 1 if $cur_period_size > $max_period_size;
	};
	return 0;
};

if (($> == 0) or ($< == 0))
{
	die "Do not run me as root! Its dangerous.\n";
};

my $DLib = DaemonLib->new
(
	logfacility 	=> 'PERIODIC'
);

my %config; # = $DLib->get_config_from_file("/usr/local/etc/sparrow.conf"); 
dbmopen(%config, "/var/db/sparrow/config", 0660)
	or $DLib->dielog("Cant open config hash (/var/db/sparrow/config): $!");

# there is nothing harmful if no debug defined by default
$DLib->debug($config{'debug'});

#my $rotated_accesslog	= $config{'rotated_accesslog'};
#my $archive_dir	= $config{'statis_archive_dir'};
#my $zero_bytecounter 	= $config{'zero_bytecounter_on_period'};
#my $squid_accesslog	= $config{'squid_access_log'};
my $squid_bin		= $config{'squid_bin'};
my $pidfile		= $config{'periodic_pidfile'};
my $access_log		= $config{'squid_access_log'};
my $period_started_at	= $config{'cur_period_start_timepoint'};
my $max_period_size	= $config{'max_period_size'};
my $archive_path = $config{"archive_basedir"};

# Now, we should not work if:
$DLib->dielog("Logfile not defined!") unless $config{'logfile'};
$DLib->logfile($config{'logfile'});
$DLib->dielog("No pidfile defined!") unless $pidfile;
$DLib->dielog("Already running.") if $DLib->checkpid_byfile($pidfile);
$DLib->dielog("Cant write PID file $pidfile! $!") unless $DLib->writepid_tofile($pidfile);
$DLib->dielog("Archive basedir ($archive_path) doesnt exists!") unless -d $archive_path;

# Database connection is mandatory
my $D = SQLayer -> new
(
        database 	=> 'DBI:Pg:dbname='.$config{'dbname'}.';host='.$config{'dbhost'},
        user 		=> $config{'dbuser'},
        password 	=> $config{'dbpassword'}
);
$DLib->dielog("Database ".$config{'dbname'}." connection failed!".$D->errstr) 
	unless $D->{'dbh'}->ping;
$D -> DEBUG(1) if $config{'debug'};

$DLib->debuglog("All checks done well. Database connected.");
my $cur_time = time;

# select all dates (except for today) available from statis_summary table
my @all_dates = 
	$D->column("SELECT stat_day FROM statis_summary ".
			"WHERE stat_day != \'today\' GROUP BY stat_day ORDER BY stat_day;");

#sub create_dir
#{
#	my $dir = shift;
#	return 1 if -d $dir;
#	mkdir ($dir)
#	or do
#	{
#		$DLib->writelog("WARNING Cant create $dir: $!\n");
#		return 0;
#	};
#	$DLib->writelog("NOTICE Archive directory $dir created.");
#	return 1;
#};

# dump statis from database to archive for selected dates
my $cur_date;
foreach $cur_date (@all_dates)
{
	# prepare archive directory for current date
#	my ($cur_year, $cur_month, $cur_day) = split(/\-/);
	my $cur_archive_path = $archive_path;
	foreach (split(/\-/, $cur_date))
	{
		$cur_archive_path .= "/".$_;
		if (-d $cur_archive_path)
		{
			$DLib->debuglog("Directory $cur_archive_path exists.");
			next;
		}; 
		mkdir ($cur_archive_path)
		or do
		{
			$DLib->writelog("WARNING Cant create $cur_archive_path: $!");
			last;
		};
		$DLib->debuglog("Directory $cur_archive_path created.");
	};
	unless (-d $cur_archive_path)
	{
		$DLib->writelog("WARNING Archive path for $cur_date not prepared! Date skipped.");
		next;
	};
	
	# select all clients for current date
	my @cur_clients = $D->column("SELECT id, sum(bytecounter) as bytes FROM statis_summary ".
							"WHERE stat_day = ".$D->quote($cur_date).
							" GROUP BY id ORDER BY bytes DESC;");
	
	# dump statis to archive
	open(DATE_ARC, ">$cur_archive_path/TOTAL.txt")
		or do
		{
			$DLib->writelog("WARNING Cant open TOTAL archive in $cur_archive_path: $!");
			next;
		};
	my $cur_id;
	foreach $cur_id (@cur_clients)
	{	
		open(CLIENT_ARC, ">$cur_archive_path/$cur_id.txt")
			or do
			{
				$DLib->writelog("WARNING [Client id = $cur_id] Cant open archive $cur_archive_path/$cur_id: $!");
				next;
			};
		print CLIENT_ARC "\$DESCR=".$D->row("SELECT descr FROM clients WHERE id = ".$D->quote($cur_id).";")."\n";
		foreach ($D->column("SELECT site ".
					"FROM statis_summary ".
					"WHERE stat_day = ".$D->quote($cur_date).
					" AND id = ".$D->quote($cur_id)." ORDER BY bytecounter DESC;"))
		{
			print CLIENT_ARC "$_ ".$D->row("SELECT bytecounter FROM statis_summary ".
					"WHERE stat_day = ".$D->quote($cur_date)." AND id = ".$D->quote($cur_id).
					" AND site = ".$D->quote($_).";")."\n";
		};
		close(CLIENT_ARC);	
		$DLib->debuglog("Date $cur_date archive for $cur_id dumped to file $cur_archive_path/$cur_id.txt.");
		print DATE_ARC 	"$cur_id " .
						$D->row("SELECT sum(bytecounter) FROM statis_summary ".
								"WHERE stat_day = ".$D->quote($cur_date).
								" AND id = ".$D->quote($cur_id).";") .
						" " .
						$D->row("SELECT descr FROM clients WHERE id = ".$D->quote($cur_id).";") . 
						"\n";
	};
	close(DATE_ARC);
	$DLib->debuglog("Date $cur_date TOTAL archive dumpted to file $cur_archive_path/TOTAL.txt.");
	
	# clear statis for current date
	if ($D->proc("DELETE FROM statis_summary WHERE stat_day = ".$D->quote($_).";"))
	{
		$DLib->writelog("Summary statis for date $_ cleared.");
	}
	else
	{
		$DLib->writelog("WARNING Table statis_summary with date $_ cleanup failed: ".$D->errstr);
	};
};

if (
	rotate_needed	(
		cur_time => $cur_time, 
		max_period_size => $max_period_size, 
		period_started_at => $period_started_at
	)
)
{
	# do log rotation with squid
	$DLib->dielog("Cant do $squid_bin -k rotate!") 
		if system("$squid_bin -k rotate");
	$DLib->debuglog("Squid logs rotated ($squid_bin -k rotate).");
	#sleep(30);

	# reset all bytecounters
	my $zeroed_counters = $D->proc("UPDATE clients SET bytecounter = 0;");
	$DLib->writelog("WARNING Cant reset bytecounters! ".$D->errstr) unless $zeroed_counters;
	$DLib->writelog("NOTICE $zeroed_counters bytecounters zeroed.") if $zeroed_counters;

	# create a new period
	$config{'cur_period_start_timepoint'} = $cur_time;
	$DLib->writelog("A new period start timepoint is: ".scalar(localtime($cur_time)));
};

unlink($pidfile);
