#!/usr/bin/perl

#       .

my $SSIDNAME = 'SPARROW_SSID';

use strict;
use CGI::Simple;
use HTML::Template;
use CGI::Session;
use POSIX qw(strftime);

CGI::Session->name($SSIDNAME);

my $cgi = CGI::Simple -> new;
my $http_header = $cgi->header(-charset=>'auto', -expires=>'+1s');

my %config; 
dbmopen(%config, "/var/db/sparrow/config", 0660)
	or die("Cant open config hash (/var/db/sparrow/config): $!\n");
my $tmpl_dir = $config{'tmpl_dir'};
my $dbname = $config{'dbname'};
my $dbuser = $config{'dbuser'};
my $dbpassword = $config{'dbpassword'};
my $dbhost = $config{'dbhost'};
my $archive_basedir = $config{'archive_basedir'};

dbmclose(%config);
die("Configuration variable tmpl_dir is empty!\n") unless $tmpl_dir;
die("Archive basedir ($archive_basedir) doesnt exists!") unless -d $archive_basedir;

my $tmpl;

my $cookie = $cgi->cookie($SSIDNAME);

my $remote_host = $ENV{'REMOTE_ADDR'};
$remote_host = $ENV{'REMOTE_HOST'} unless $remote_host;

#     IP  - 
unless ($remote_host)
{
	$tmpl = HTML::Template->new(filename => $tmpl_dir.'/not_traceable.html');
	print STDOUT $http_header . $tmpl->output;
	exit;
};

my $cgi_ss = CGI::Session->load($cookie)
	or die CGI::Session->errstr();
my $ident = $cgi_ss->param('ident');

#  
my $reauth_needed = 0;
$reauth_needed = 1 if $cgi_ss->is_empty;
$reauth_needed = 1 if $cgi_ss->is_expired;
$reauth_needed = 1 if $cgi_ss->param('remote_host')	ne $remote_host;
$reauth_needed = 1 unless $ident;

#  ,    
if ($reauth_needed)
{
	$tmpl = HTML::Template->new(filename => $tmpl_dir.'/reauth_needed.html');
	print STDOUT $http_header.$tmpl->output;
	exit;
};

use SQLayer;
my $D = SQLayer -> new
(
	database    => 'DBI:Pg:dbname='.$dbname.';host='.$dbhost,
	user        => $dbuser,
	password    => $dbpassword
);

die "No database connection: ".$D->errstr."\n" unless $D->{'dbh'}->ping();

#   
my $is_admin = $D->row("SELECT is_admin FROM clients WHERE id = ".$D->quote($ident).";");

#     
my $req_year = $cgi->param('arc_year');
if ($req_year and ($req_year !~ /^\d{4}$/))
{
	die "Requested archive year is invalid ($req_year).\n";
};
my $req_month = $cgi->param('arc_month');
if ($req_month and ($req_month !~ /^\d{2}$/))
{
	die "Requested archive month is invalid ($req_month).\n";
};
my $req_day = $cgi->param('arc_day');
if ($req_day and ($req_day !~ /^\d{2}$/))
{
	die "Requested archive day is invalid ($req_day).\n";
};

# get_href_block_for_tmpl
# (
# 		item => 'arc_year', 
# 		req_item => '2009', 
# 		listdir => '/usr/local/www/archive'
# );
sub get_href_block_for_tmpl
{
	my %HKeys = @_;
	my $item = $HKeys{'item'};
	my $req_item = $HKeys{'req_item'}; # use last, if empty
	my $listdir = $HKeys{'listdir'};
#	warn "item = $item, req_item = $req_item, listdir = $listdir.\n";
	opendir (LISTDIR, $listdir)	
		or die "Cant open $listdir for reading: $!\n";
	my @href_block;
	my $last_item = 0; # will be used, if no item requested at all
	while (my $dir_entry = readdir (LISTDIR))
	{
		next unless $dir_entry =~ /^\d+$/; # only numeric directories wanted
#		warn "processing directory $listdir/$dir_entry...\n";
		unless (-d "$listdir/$dir_entry")
		{
#			warn "$listdir/$dir_entry is not a directory!\n";
			next;
		};
		my %select_str;
		$select_str{$item} = $dir_entry; # value for <option value="x"> 
		if ($req_item)
		{
			if ($req_item eq $dir_entry)
			{
				$select_str{'selected'} = 'selected';
				$last_item = 'none'; # last item does not matter, if some item requested
#				warn "requested $item ($req_item) founded.\n";
			}
			else
			{
				$select_str{'selected'} = ''; # should be used as <option value="x" <TMPL_VAR NAME=selected> >
#				warn "requested $item ($req_item) not founded yet (this directory is $dir_entry).\n";
			};
		}
		else
		{
			$select_str{'selected'} = '';
#			warn "no item requested. this directory is $dir_entry.\n";
		};
		push(@href_block, \%select_str);
		if ($last_item eq 'none')
		{
#			warn "requested $item ($req_item) already founded, so last directory does not matter. Processing next directory\n";
			next;
		};
		$last_item = $dir_entry if $dir_entry > $last_item;
#		warn "requested $item ($req_item) is not founded yet, so last_item is $last_item now (this directory is $dir_entry).\n";
	};
	closedir (LISTDIR);
	return undef if @href_block == 0;
	if ($last_item ne 'none')
	{
		die "Non-existent $item ($req_item) requested!\n" if $req_item;
	}
	else
	{
		$last_item = '';
	};
	return (\@href_block, $last_item);
};

#   
my $display_archive_form;
my ($last_month, $last_day, $month_list_ahref, $day_list_ahref);
my ($year_list_ahref, $last_year) = get_href_block_for_tmpl
(
	item => 'arc_year', req_item => $req_year, listdir => $archive_basedir
);
if ($year_list_ahref)
{
	#    .
	#     ,       -    periodic 
	$display_archive_form = 1;
	$req_year = $last_year unless $req_year;
	($month_list_ahref, $last_month) = get_href_block_for_tmpl
	(
		item => 'arc_month', req_item => $req_month, 
		listdir => "$archive_basedir/$req_year"
	);
	die "Empty directory $archive_basedir/$req_year!\n" unless $month_list_ahref;
	$req_month = $last_month unless $req_month;

	($day_list_ahref, $last_day) = get_href_block_for_tmpl
	(
		item => 'arc_day', req_item => $req_day, 
		listdir => "$archive_basedir/$req_year/$req_month"
	);
	die "Empty directory $archive_basedir/$req_year/$req_month!\n" unless $day_list_ahref;
	$req_day = $last_day unless $req_day;
};
my $arc_workpath = "$archive_basedir/$req_year/$req_month/$req_day";

#    
my $req_ident = $cgi->param('login');
if ($req_ident)
{
	die "Invalid login requested! ($req_ident)\n"
		unless $req_ident =~ /[\d\w\_\-\.]+/;
};

#       
$req_ident = $ident unless $is_admin;

my $today = strftime("%Y-%m-%d", localtime(time)); # ex: 1999-01-08 -- ISO 8601

$tmpl = HTML::Template->new(filename => $tmpl_dir.'/statis.html');
$tmpl->param(today => $today);

if ($display_archive_form)
{
	#       
	$tmpl->param(year_list => $year_list_ahref);
	$tmpl->param(month_list => $month_list_ahref);
	$tmpl->param(day_list => $day_list_ahref);
	$tmpl->param(last_year => $last_year);
	$tmpl->param(last_month => $last_month);
	$tmpl->param(last_day => $last_day);
	#      
	$tmpl->param(cur_arc_year => $req_year);
	$tmpl->param(cur_arc_month => $req_month);
	$tmpl->param(cur_arc_day => $req_day);
};
$tmpl->param(display_archive_form => $display_archive_form);


if ($req_ident or ($req_ident eq '0'))
{
	#  ,       
	if ($cgi->param('arc_year'))
	{
		#   
		unless(open (ARC_WORK, "$arc_workpath/$req_ident.txt"))
		{
			#die "Cant open directory $arc_workpath/$req_ident: $!\n";
			$tmpl->param(null_data => 1);
		}
		else	
		{
			warn "workpath = $arc_workpath (id = $req_ident)\n";
			my @arc_list;
			my $descr;
			while (<ARC_WORK>)
			{
				chomp;
				if (/^\$DESCR\=(.*)/)
				{
					$descr = $1;
					next;
				};
				my %arc_str;
				($arc_str{'site'}, $arc_str{'bytes'}) = split(/\s+/);
				unless ($arc_str{'bytes'} =~ /^\d+$/)
				{
					warn("Bytecounter for login ".$arc_str{'login'}." is not numeric!\n");
					next;
				};
				$arc_str{'bytes'} = sprintf('%.2f',($arc_str{'bytes'} / 1000000));
				push (@arc_list, \%arc_str);
			};
			close (ARC_WORK);
			$tmpl->param(site_stat => \@arc_list);
			$tmpl->param(descr => $descr);
			$tmpl->param(working_with_archive => 1);
		}
	}
	else
	{
		#      
		$tmpl->param
		(
			site_stat => $D->row_hash("SELECT site, ".
				"\'\' || (bytecounter / 1000000) || \'.\' || to_char(((bytecounter % 1000000)/10000),\'09\') || \'\' as bytes ".
#				"trunc((bytecounter / 1000000), 1) as bytes ".
				"FROM statis_summary WHERE id = ".$D->quote($req_ident).
				"AND stat_day = \'today\' ORDER BY bytecounter DESC;")
		);
		$tmpl->param(descr => $D->row("SELECT descr FROM clients WHERE id = ".$D->quote($req_ident)));
	};
	$tmpl->param(req_ident => $req_ident);
	print STDOUT $http_header . $tmpl->output;
	exit;
}
else
{
	#    -       
	if ($cgi->param('arc_year'))
	{
		#   
		open (ARC_WORK, "$arc_workpath/TOTAL.txt")
			or die "Cant open directory $arc_workpath/TOTAL.txt: $!\n";
		warn "workpath = $arc_workpath (TOTAL)\n";
		my @arc_list;
		while (<ARC_WORK>)
		{
			chomp;
			my %arc_str;
			/^(\S+)\s+(\S+)\s(.*)$/;
			$arc_str{'login'} = $1;
			$arc_str{'bytes'} = $2;
			$arc_str{'descr'} = $3;
			unless ($arc_str{'bytes'} =~ /^\d+$/)
			{
				warn("Bytecounter for login ".$arc_str{'login'}." is not numeric!\n");
				next;
			};
			$arc_str{'bytes'} = sprintf('%.2f',($arc_str{'bytes'} / 1000000));
			#   ,     client_id    
			$arc_str{'req_year'} = $req_year;
			$arc_str{'req_month'} = $req_month;
			$arc_str{'req_day'} = $req_day;
			push (@arc_list, \%arc_str);
		};
		$tmpl->param(user_stat => \@arc_list);
		$tmpl->param(working_with_archive => 1);
	}
	else
	{
		#      
		$tmpl->param
		(
			user_stat => $D->row_hash("SELECT clients.id as login, ".
				"round((sum(statis_summary.bytecounter) / 1000000), 2) as bytes, ".
				"clients.descr as descr ".
				"FROM statis_summary, clients WHERE clients.id = statis_summary.id ".
				"AND stat_day = \'today\' GROUP BY clients.id, clients.descr ORDER BY bytes DESC;")
		);
	};
	print STDOUT $http_header . $tmpl->output;
	exit;
};


